data_type_prompt = """
This is an array of Apache Atlas entities: {{entities}}. 

Review entities with the "typeName" of "column" and the "typeName" of "field"
and compare the entity name to this list of self-describing data types:

[ 
    "aba",
    "add_provider",
    "address",
    "administrative_unit",
    "am_pm",
    "android_platform_token",
    "ascii_company_email",
    "ascii_email",
    "ascii_free_email",
    "ascii_safe_email",
    "bank_country",
    "basic_phone_number",
    "bban",
    "binary",
    "boolean",
    "bothify",
    "bs",
    "building_number",
    "cache_pattern",
    "catch_phrase",
    "century",
    "chrome",
    "city",
    "city_prefix",
    "city_suffix",
    "color",
    "color_hsl",
    "color_hsv",
    "color_name",
    "color_rgb",
    "color_rgb_float",
    "company",
    "company_email",
    "company_suffix",
    "coordinate",
    "country",
    "country_calling_code",
    "country_code",
    "credit_card_expire",
    "credit_card_full",
    "credit_card_number",
    "credit_card_provider",
    "credit_card_security_code",
    "cryptocurrency",
    "cryptocurrency_code",
    "cryptocurrency_name",
    "csv",
    "currency",
    "currency_code",
    "currency_name",
    "currency_symbol",
    "current_country",
    "current_country_code",
    "date",
    "date_between",
    "date_between_dates",
    "date_object",
    "date_of_birth",
    "date_this_century",
    "date_this_decade",
    "date_this_month",
    "date_this_year",
    "date_time",
    "date_time_ad",
    "date_time_between",
    "date_time_between_dates",
    "date_time_this_century",
    "date_time_this_decade",
    "date_time_this_month",
    "date_time_this_year",
    "day_of_month",
    "day_of_week",
    "del_arguments",
    "dga",
    "domain_name",
    "domain_word",
    "dsv",
    "ean",
    "ean13",
    "ean8",
    "ein",
    "email",
    "emoji",
    "enum",
    "factories",
    "file_extension",
    "file_name",
    "file_path",
    "firefox",
    "first_name",
    "first_name_female",
    "first_name_male",
    "first_name_nonbinary",
    "fixed_width",
    "format",
    "free_email",
    "free_email_domain",
    "future_date",
    "future_datetime",
    "generator_attrs",
    "get_arguments",
    "get_formatter",
    "get_providers",
    "get_words_list",
    "hex_color",
    "hexify",
    "hostname",
    "http_method",
    "http_status_code",
    "iana_id",
    "iban",
    "image",
    "image_url",
    "internet_explorer",
    "invalid_ssn",
    "ios_platform_token",
    "ipv4",
    "ipv4_network_class",
    "ipv4_private",
    "ipv4_public",
    "ipv6",
    "isbn10",
    "isbn13",
    "iso8601",
    "items",
    "itin",
    "job",
    "json",
    "json_bytes",
    "language_code",
    "language_name",
    "last_name",
    "last_name_female",
    "last_name_male",
    "last_name_nonbinary",
    "latitude",
    "latlng",
    "lexify",
    "license_plate",
    "linux_platform_token",
    "linux_processor",
    "local_latlng",
    "locale",
    "locales",
    "localized_ean",
    "localized_ean13",
    "localized_ean8",
    "location_on_land",
    "longitude",
    "mac_address",
    "mac_platform_token",
    "mac_processor",
    "md5",
    "military_apo",
    "military_dpo",
    "military_ship",
    "military_state",
    "mime_type",
    "month",
    "month_name",
    "msisdn",
    "name",
    "name_female",
    "name_male",
    "name_nonbinary",
    "nic_handle",
    "nic_handles",
    "null_boolean",
    "numerify",
    "opera",
    "optional",
    "paragraph",
    "paragraphs",
    "parse",
    "passport_dates",
    "passport_dob",
    "passport_full",
    "passport_gender",
    "passport_number",
    "passport_owner",
    "password",
    "past_date",
    "past_datetime",
    "phone_number",
    "port_number",
    "postalcode",
    "postalcode_in_state",
    "postalcode_plus4",
    "postcode",
    "postcode_in_state",
    "prefix",
    "prefix_female",
    "prefix_male",
    "prefix_nonbinary",
    "pricetag",
    "profile",
    "provider",
    "providers",
    "psv",
    "pybool",
    "pydecimal",
    "pydict",
    "pyfloat",
    "pyint",
    "pyiterable",
    "pylist",
    "pyobject",
    "pyset",
    "pystr",
    "pystr_format",
    "pystruct",
    "pytimezone",
    "pytuple",
    "random",
    "random_choices",
    "random_digit",
    "random_digit_above_two",
    "random_digit_not_null",
    "random_digit_not_null_or_empty",
    "random_digit_or_empty",
    "random_element",
    "random_elements",
    "random_int",
    "random_letter",
    "random_letters",
    "random_lowercase_letter",
    "random_number",
    "random_sample",
    "random_uppercase_letter",
    "randomize_nb_elements",
    "rgb_color",
    "rgb_css_color",
    "ripe_id",
    "safari",
    "safe_color_name",
    "safe_domain_name",
    "safe_email",
    "safe_hex_color",
    "sbn9",
    "secondary_address",
    "seed",
    "seed_instance",
    "seed_locale",
    "sentence",
    "sentences",
    "set_arguments",
    "set_formatter",
    "sha1",
    "sha256",
    "simple_profile",
    "slug",
    "ssn",
    "state",
    "state_abbr",
    "street_address",
    "street_name",
    "street_suffix",
    "suffix",
    "suffix_female",
    "suffix_male",
    "suffix_nonbinary",
    "swift",
    "swift11",
    "swift8",
    "tar",
    "text",
    "texts",
    "time",
    "time_delta",
    "time_object",
    "time_series",
    "timezone",
    "tld",
    "tsv",
    "unique",
    "unix_device",
    "unix_partition",
    "unix_time",
    "upc_a",
    "upc_e",
    "uri",
    "uri_extension",
    "uri_page",
    "uri_path",
    "url",
    "user_agent",
    "user_name",
    "uuid4",
    "vin",
    "weights",
    "windows_platform_token",
    "word",
    "words",
    "xml",
    "year",
    "zip",
    "zipcode",
    "zipcode_in_state",
    "zipcode_plus4"
]

For each entity where there is any potential that the 
entity is related to a data type, assign the more precise data type.

Return a dictionary in the format:

{
    "<entity qualified name>": "<data type>"
}

"""



def get_collection(subgraph_data_connector, collection_name):
    collection = [item for item in subgraph_data_connector['schema']['collections'] if item['name'] == collection_name][
        0]
    return collection


def create_collection(subgraph_data_connector, collection_name, supergraph_name, subgraph_name, column_ids, entities):
    collection = get_collection(subgraph_data_connector, collection_name)
    qualified_name = f"{supergraph_name}.{subgraph_name}.{collection_name}"

    entities.append({
        "typeName": "collection",
        "attributes": {
            "name": collection_name,
            "qualifiedName": qualified_name,
            "primary_keys": list(collection['uniqueness_constraints']['PK']['unique_columns']),
            "columns": [{"typeName": "column", "uniqueAttributes": {"qualifiedName": item}} for item in
                        column_ids]
        }
    })

    return qualified_name



import logging
from ddnatlas.business_metadata import add_business_metadata
from ddnatlas.descriptions import add_descriptions
from ddnatlas.generate_subgraph import generate_subgraph
from ddnatlas.glossary import create_glossary
from ddnatlas.parse_files import parse_files
from ddnatlas.relationships import create_relationships
from ddnatlas.update_entities import update_entities
import os
from typing import List, Optional


def update_supergraph_metadata(include: Optional[List[str]], exclude: Optional[List[str]]):

    supergraph_directory = os.getenv('SUPERGRAPH')

    logging.info("Parsing supergraph configuration files...")
    parsed_data = parse_files(supergraph_directory)

    logging.info("Filtering parsed data...")
    parsed_data = [item for item in parsed_data if isinstance(item, dict) and item.get('kind') is not None]

    logging.info("Generating supergraph object...")
    supergraph = [item for item in parsed_data if item.get('kind') == 'Supergraph'][0]

    logging.info("Generating subgraph names...")
    subgraph_names = [item.split('/')[0] for item in supergraph['definition']['subgraphs'] if
                      not item.startswith("globals/")]

    logging.info("Generating subgraph objects...")
    subgraphs = [item for item in parsed_data if
                 item.get('kind') == 'Subgraph' and item['definition']['name'] in subgraph_names]

    logging.info("Generating object types objects...")
    object_types = [item for item in parsed_data if item.get('kind') == 'ObjectType']

    logging.info("Generating data connector links objects...")
    data_connector_links = [item for item in parsed_data if item.get('kind') == 'DataConnectorLink']

    logging.info("Generating data connector objects...")
    data_connectors = [item for item in parsed_data if item.get('kind') == 'Connector']

    logging.info("Generating model objects...")
    models = [item for item in parsed_data if item.get('kind') == 'Model']

    logging.info("Generating relationship objects...")
    relationships = [item for item in parsed_data if item.get('kind') == 'Relationship']

    all_entities = [
        {
            "typeName": "transform",
            "attributes": {
                "name": "straight_through",
                "qualifiedName": f"{supergraph['definition'].get('name', 'enterprise')}.straight_through",
                "transformation": "NONE",
                "explanation": "No intentional logic was invoked to modify this value. Platforms may have "
                               "serialization and deserialization methods that are inherently invoked and may alter"
                               "the value."
            }
        }
    ]
    update_entities(all_entities, include, exclude)

    logging.info("Analyzing subgraphs...")
    for subgraph in subgraphs:
        all_entities.extend(
            generate_subgraph(supergraph, subgraph, data_connectors, data_connector_links, object_types, models,
                              include, exclude))

    subgraph_ids = [item['attributes']['qualifiedName'] for item in all_entities if item['typeName'] == 'subgraph']

    logging.info("Adding supergraph...")
    supergraph_entity = [{
        "typeName": "supergraph",
        "attributes": {
            "name": "enterprise",
            "console_url": "",
            "grpc_service_url": "",
            "openapi_ui_url": "",
            "openapi_service_url": "",
            "graphql_service_url": "http://localhost:3000/graphql",
            "qualifiedName": "enterprise",
            "description": "The enterprise supergraph.",
            "domains": [{"typeName": "subgraph", "uniqueAttributes": {"qualifiedName": item}} for item in
                        subgraph_ids]
        }
    }]
    entities = all_entities + supergraph_entity
    supergraph_name = supergraph['definition'].get('name', 'enterprise')

    try:
        remove_entities = add_descriptions(entities, include, exclude)
        logging.info("Updating entities..")
        update_entities(entities, include, exclude, force_updates=remove_entities)
    except Exception as e:
        logging.error(e)

    logging.info("Adding business metadata")
    try:
        add_business_metadata(supergraph_name, entities, include, exclude)
    except Exception as e:
        logging.error(e)

    logging.info("Adding business glossary")
    try:
        create_glossary(supergraph_name, entities, include, exclude)
    except Exception as e:
        logging.error(e)

    logging.info("Creating relationships..")
    try:
        create_relationships(relationships, object_types, entities)
    except Exception as e:
        logging.error(e)

    logging.info("Success!")



def create_data_connector(supergraph_name, subgraph_name, connector_name, subgraph_data_connector, scalar_ids,
                          collection_ids, entities):
    qualified_name = f"{supergraph_name}.{subgraph_name}.{connector_name}"
    entities.append({
        "typeName": "data_connector",
        "attributes": {
            "qualifiedName": qualified_name,
            "name": connector_name,
            "subgraph": subgraph_data_connector['definition']['subgraph'],
            "source": subgraph_data_connector['definition']['source'],
            "env_mapping": subgraph_data_connector['definition']['envMapping'],
            "scalar_types": [{"typeName": "scalar", "uniqueAttributes": {"qualifiedName": item}} for item in
                             scalar_ids],
            "functions": [item.get('name') for item in subgraph_data_connector['schema']['functions']],
            "procedures": [item.get('name') for item in subgraph_data_connector['schema']['procedures']],
            "collections": [{"typeName": "collection", "uniqueAttributes": {"qualifiedName": item}} for item in
                            collection_ids],
        }
    })
    return qualified_name



import json
import os

from apache_atlas.client.base_client import AtlasClient

from ddnatlas.anthropic_prompt_data_type import data_type_prompt
from ddnatlas.camel_to_title import camel_to_title
from ddnatlas.claude import process_json_with_claude


def add_business_metadata(supergraph, entities, include, exclude):
    url = os.getenv('ATLAS_URL')
    username = os.getenv('ATLAS_USERNAME')
    password = os.getenv('ATLAS_PASSWORD')

    if include and 'business_metadata' not in include:
        return
    if exclude and 'business_metadata' in exclude:
        return

    api_key = os.getenv('ANTHROPIC_API_KEY')
    if api_key:
        testing_types = process_json_with_claude(
            data_type_prompt
            .replace('{{entities}}', json.dumps(entities))
            .replace('{{supergraph}}', supergraph),
            api_key
        )
    else:
        testing_types = {}

    client = AtlasClient(host=url, auth=(username, password))
    for entity in entities:
        if entity.get('typeName') and entity['attributes'].get('qualifiedName') and entity['attributes'].get(
                'name') and entity.get('typeName') in ["field", "column", "query", "collection", "subgraph",
                                                       "supergraph", "object_type"]:
            qualified_name = entity['attributes']['qualifiedName']
            testing_type = testing_types.get(qualified_name)
            technical_name = entity['attributes']['name']
            business_name = camel_to_title(technical_name.replace('_', ' '))
            result = client.discovery.attribute_search(type_name='Asset', attr_name='qualifiedName',
                                                       attr_value_prefix=qualified_name, limit=1, offset=0)
            guid = result['entities'][0]['guid']
            if guid:
                full_entity = client.entity.get_entity_by_guid(guid=guid)
                business_attributes = full_entity['entity'].get('businessAttributes', {})
                data_analysis = business_attributes.get('data_analysis', {})
                business_names = data_analysis.get('businessNames', [])
                business_names.append(business_name)
                business_names = list(set(business_names))
                data_analysis["testingType"] = data_analysis.get('testingType', testing_type)
                data_analysis["businessNames"] = business_names
                business_attributes['data_analysis'] = data_analysis

                try:
                    client.entity.add_or_update_business_attributes(entity_guid=guid, is_overwrite=True,
                                                                    business_attributes=business_attributes)
                except Exception as e:
                    pass



import json
import logging
import os
from apache_atlas.client.base_client import AtlasClient

from ddnatlas.supergraph_types import supergraph_types


def generate_supergraph_types():

    atlas_url = os.getenv('ATLAS_URL')
    username = os.getenv('ATLAS_USERNAME')
    password = os.getenv('ATLAS_PASSWORD')

    logging.info('Generating supergraph types')

    client = AtlasClient(atlas_url, (username, password))

    try:
        response = client.typedef.create_atlas_typedefs(supergraph_types)
        logging.info('Request execution completed.')
        print(response)
    except Exception as e:
        logging.info('Request execution completed with errors.')
        logging.error(e)



import json
import os

from apache_atlas.model.glossary import AtlasGlossaryTerm, AtlasGlossary, AtlasGlossaryHeader, AtlasGlossaryCategory
from apache_atlas.model.instance import AtlasClassification
from apache_atlas.model.relationship import AtlasRelationship

from ddnatlas.anthropic_prompt import glossary_prompt
from ddnatlas.anthropic_prompt_data_type import data_type_prompt
from ddnatlas.claude import process_json_with_claude
from apache_atlas.client.base_client import AtlasClient, type_coerce

from ddnatlas.supergraph_types import enum_list


def create_glossary(supergraph, entities, include=None, exclude=None):
    url = os.getenv('ATLAS_URL')
    username = os.getenv('ATLAS_USERNAME')
    password = os.getenv('ATLAS_PASSWORD')
    api_key = os.getenv('ANTHROPIC_API_KEY')

    if not api_key:
        return
    if include and 'glossary' not in include:
        return
    if exclude and 'glossary' in exclude:
        return

    client = AtlasClient(host=url, auth=(username, password))
    glossary = process_json_with_claude(
        glossary_prompt
        .replace('{{entities}}', json.dumps(entities))
        .replace('{{supergraph}}', supergraph),
        api_key
    )
    glossaries = client.glossary.get_all_glossaries()
    existing_glossary = [item for item in glossaries if
                         item.get('qualifiedName') == glossary['glossary'].get('qualifiedName')]
    if not existing_glossary:
        atlas_glossary = AtlasGlossary(glossary['glossary'])
        new_glossary = client.glossary.create_glossary(glossary=atlas_glossary)
    else:
        new_glossary = existing_glossary[0]

    glossary_guid = new_glossary.get('guid')
    glossary_name = new_glossary.get('name')

    for category in glossary['categories']:
        anchor = AtlasGlossaryHeader({'glossaryGuid': glossary_guid, 'displayText': glossary_name})
        atlas_category = AtlasGlossaryCategory({
            'name': category.get('name'),
            'qualifiedName': category.get('qualifiedName'),
            'shortDescription': category.get('shortDescription'),
            'longDescription': category.get('longDescription'),
            'anchor': anchor})
        try:
            client.glossary.create_glossary_category(atlas_category)
        except Exception:
            pass

    new_glossary = client.glossary.get_glossary_by_guid(glossary_guid=glossary_guid)
    terms = glossary['terms']
    for term in terms:
        for category in term['categories']:
            guid = category['categoryGuid']
            temp_guid = [item for item in glossary['categories'] if item['guid'] == guid]
            if temp_guid:
                name = temp_guid[0]['name']
                new_guid = [item['categoryGuid'] for item in new_glossary['categories'] if item['displayText'] == name]
                if new_guid:
                    anchor = AtlasGlossaryHeader(
                        {'glossaryGuid': glossary_guid, 'displayText': glossary_name})
                    atlas_term = AtlasGlossaryTerm({
                        'name': term.get('name'),
                        'qualifiedName': term.get('qualifiedName'),
                        'shortDescription': term.get('shortDescription'),
                        'longDescription': term.get('longDescription'),
                        'examples': term.get('examples'),
                        'abbreviation': term.get('abbreviation'),
                        'usage': term.get('usage'),
                        'categories': [{'categoryGuid': new_guid[0]}],
                        'anchor': anchor})
                    try:
                        client.glossary.create_glossary_term(atlas_term)
                    except Exception:
                        pass

    new_glossary = client.glossary.get_glossary_ext_info(glossary_guid=glossary_guid)
    relationships = glossary['relationships']
    for relationship in relationships:
        temp_guid = relationship['end1']['guid']
        old_name = [item['name'] for item in glossary['terms'] if item['guid'] == temp_guid]
        if old_name:
            old_name = old_name[0]
            new_guid = [item['guid'] for item in new_glossary['termInfo'].values() if item['name'] == old_name]
            if new_guid:
                new_guid = new_guid[0]
                relationship['end1']['guid'] = new_guid
                atlas_relationship = type_coerce(relationship, AtlasRelationship)
                try:
                    client.relationship.create_relationship(relationship=atlas_relationship)
                except Exception:
                    pass

    for qualified_name, classifications in glossary['classifications'].items():
        result = client.discovery.attribute_search(type_name='Asset', attr_name='qualifiedName',
                                                   attr_value_prefix=qualified_name, limit=1, offset=0)
        guid = result['entities'][0]['guid']
        if guid:
            atlas_classification = [AtlasClassification({"typeName": item}) for item in classifications]
            try:
                client.entity.add_classifications_by_guid(guid=guid, classifications=atlas_classification)
            except Exception:
                pass



import os

from ddnatlas.execute_request import execute_request


def get_entities():
    atlas_url = os.getenv('ATLAS_URL')
    username = os.getenv('ATLAS_USERNAME')
    password = os.getenv('ATLAS_PASSWORD')
    response = execute_request("get", f"{atlas_url}/api/atlas/v2/search/dsl?limit=1000&offset=0&typeName=Asset",
                               username=username, password=password).json()
    return response['entities']




from ddnatlas import primitives


def create_fields(supergraph_name, subgraph_name, object_name, object_type, entities):
    field_ids = []
    for field in object_type['definition']['fields']:
        field_name = field['name']
        column_id = ""
        for mapping in object_type['definition']['dataConnectorTypeMapping']:
            elem = mapping['fieldMapping'][field_name]
            column_id = f"{supergraph_name}.{subgraph_name}.{mapping['dataConnectorObjectType']}.{elem['column']['name']}"
        qualified_name = f"{supergraph_name}.{subgraph_name}.{object_name}.{field_name}"
        field_ids.append(qualified_name)
        entities.append({
            "typeName": "field",
            "attributes": {
                "name": field_name,
                "qualifiedName": qualified_name,
                "primitive": primitives.apache_primitives[field['type'].replace('!', '')],
                "column": {
                    "typeName": "column",
                    "uniqueAttributes": {
                        "qualifiedName": column_id
                    },
                }
            }
        })
    return field_ids



enum_list = [
    "aba",
    "add_provider",
    "address",
    "administrative_unit",
    "am_pm",
    "android_platform_token",
    "ascii_company_email",
    "ascii_email",
    "ascii_free_email",
    "ascii_safe_email",
    "bank_country",
    "basic_phone_number",
    "bban",
    "binary",
    "boolean",
    "bothify",
    "bs",
    "building_number",
    "cache_pattern",
    "catch_phrase",
    "century",
    "chrome",
    "city",
    "city_prefix",
    "city_suffix",
    "color",
    "color_hsl",
    "color_hsv",
    "color_name",
    "color_rgb",
    "color_rgb_float",
    "company",
    "company_email",
    "company_suffix",
    "coordinate",
    "country",
    "country_calling_code",
    "country_code",
    "credit_card_expire",
    "credit_card_full",
    "credit_card_number",
    "credit_card_provider",
    "credit_card_security_code",
    "cryptocurrency",
    "cryptocurrency_code",
    "cryptocurrency_name",
    "csv",
    "currency",
    "currency_code",
    "currency_name",
    "currency_symbol",
    "current_country",
    "current_country_code",
    "date",
    "date_between",
    "date_between_dates",
    "date_object",
    "date_of_birth",
    "date_this_century",
    "date_this_decade",
    "date_this_month",
    "date_this_year",
    "date_time",
    "date_time_ad",
    "date_time_between",
    "date_time_between_dates",
    "date_time_this_century",
    "date_time_this_decade",
    "date_time_this_month",
    "date_time_this_year",
    "day_of_month",
    "day_of_week",
    "del_arguments",
    "dga",
    "domain_name",
    "domain_word",
    "dsv",
    "ean",
    "ean13",
    "ean8",
    "ein",
    "email",
    "emoji",
    "enum",
    "factories",
    "file_extension",
    "file_name",
    "file_path",
    "firefox",
    "first_name",
    "first_name_female",
    "first_name_male",
    "first_name_nonbinary",
    "fixed_width",
    "format",
    "free_email",
    "free_email_domain",
    "future_date",
    "future_datetime",
    "generator_attrs",
    "get_arguments",
    "get_formatter",
    "get_providers",
    "get_words_list",
    "hex_color",
    "hexify",
    "hostname",
    "http_method",
    "http_status_code",
    "iana_id",
    "iban",
    "image",
    "image_url",
    "internet_explorer",
    "invalid_ssn",
    "ios_platform_token",
    "ipv4",
    "ipv4_network_class",
    "ipv4_private",
    "ipv4_public",
    "ipv6",
    "isbn10",
    "isbn13",
    "iso8601",
    "items",
    "itin",
    "job",
    "json",
    "json_bytes",
    "language_code",
    "language_name",
    "last_name",
    "last_name_female",
    "last_name_male",
    "last_name_nonbinary",
    "latitude",
    "latlng",
    "lexify",
    "license_plate",
    "linux_platform_token",
    "linux_processor",
    "local_latlng",
    "locale",
    "locales",
    "localized_ean",
    "localized_ean13",
    "localized_ean8",
    "location_on_land",
    "longitude",
    "mac_address",
    "mac_platform_token",
    "mac_processor",
    "md5",
    "military_apo",
    "military_dpo",
    "military_ship",
    "military_state",
    "mime_type",
    "month",
    "month_name",
    "msisdn",
    "name",
    "name_female",
    "name_male",
    "name_nonbinary",
    "nic_handle",
    "nic_handles",
    "null_boolean",
    "numerify",
    "opera",
    "optional",
    "paragraph",
    "paragraphs",
    "parse",
    "passport_dates",
    "passport_dob",
    "passport_full",
    "passport_gender",
    "passport_number",
    "passport_owner",
    "password",
    "past_date",
    "past_datetime",
    "phone_number",
    "port_number",
    "postalcode",
    "postalcode_in_state",
    "postalcode_plus4",
    "postcode",
    "postcode_in_state",
    "prefix",
    "prefix_female",
    "prefix_male",
    "prefix_nonbinary",
    "pricetag",
    "profile",
    "provider",
    "providers",
    "psv",
    "pybool",
    "pydecimal",
    "pydict",
    "pyfloat",
    "pyint",
    "pyiterable",
    "pylist",
    "pyobject",
    "pyset",
    "pystr",
    "pystr_format",
    "pystruct",
    "pytimezone",
    "pytuple",
    "random",
    "random_choices",
    "random_digit",
    "random_digit_above_two",
    "random_digit_not_null",
    "random_digit_not_null_or_empty",
    "random_digit_or_empty",
    "random_element",
    "random_elements",
    "random_int",
    "random_letter",
    "random_letters",
    "random_lowercase_letter",
    "random_number",
    "random_sample",
    "random_uppercase_letter",
    "randomize_nb_elements",
    "rgb_color",
    "rgb_css_color",
    "ripe_id",
    "safari",
    "safe_color_name",
    "safe_domain_name",
    "safe_email",
    "safe_hex_color",
    "sbn9",
    "secondary_address",
    "seed",
    "seed_instance",
    "seed_locale",
    "sentence",
    "sentences",
    "set_arguments",
    "set_formatter",
    "sha1",
    "sha256",
    "simple_profile",
    "slug",
    "ssn",
    "state",
    "state_abbr",
    "street_address",
    "street_name",
    "street_suffix",
    "suffix",
    "suffix_female",
    "suffix_male",
    "suffix_nonbinary",
    "swift",
    "swift11",
    "swift8",
    "tar",
    "text",
    "texts",
    "time",
    "time_delta",
    "time_object",
    "time_series",
    "timezone",
    "tld",
    "tsv",
    "unique",
    "unix_device",
    "unix_partition",
    "unix_time",
    "upc_a",
    "upc_e",
    "uri",
    "uri_extension",
    "uri_page",
    "uri_path",
    "url",
    "user_agent",
    "user_name",
    "uuid4",
    "vin",
    "weights",
    "windows_platform_token",
    "word",
    "words",
    "xml",
    "year",
    "zip",
    "zipcode",
    "zipcode_in_state",
    "zipcode_plus4"
]
element_defs = [{"value": val, "ordinal": idx} for idx, val in enumerate(enum_list)]
supergraph_types = {
    "enumDefs": [
        {
            "name": "testingTypeEnumName",
            "serviceType": "supergraph",
            "description": "Enum for testing type",
            "typeVersion": "1.0",
            "elementDefs": element_defs
        },
        {
            "elementDefs": [
                {
                    "value": "NONE",
                    "ordinal": 0
                },
                {
                    "value": "SUM",
                    "ordinal": 1
                },
                {
                    "value": "AVERAGE",
                    "ordinal": 2
                },
                {
                    "value": "LIST",
                    "ordinal": 3
                },
                {
                    "value": "MAX",
                    "ordinal": 4
                },
                {
                    "value": "MIN",
                    "ordinal": 5
                },
                {
                    "value": "NOT_NULL",
                    "ordinal": 6
                },
                {
                    "value": "IS_NULL",
                    "ordinal": 7
                },
                {
                    "value": "BOOLEAN_COMPARISON",
                    "ordinal": 8
                },
                {
                    "value": "OTHER",
                    "ordinal": 9999
                }
            ],
            "name": "transform_enum",
            "serviceType": "supergraph",
            "description": "Description of transformation from one primitive to another",
            "typeVersion": "1.0"
        }
    ],
    "classificationDefs": [
        {
            "category": "CLASSIFICATION",
            "name": "PII",
            "description": "Personally Identifiable Information that can be used to identify an individual, either on "
                           "its own or when combined with other information. This includes names, addresses, "
                           "social security numbers, email addresses, and phone numbers. PII requires stringent "
                           "protection measures to prevent identity theft and ensure privacy",
            "superTypes": [],
            "attributeDefs": []
        },
        {
            "category": "CLASSIFICATION",
            "name": "Confidential",
            "description": "Sensitive information that, if disclosed, could harm the organization or individuals. "
                           "This data requires strict access controls. Examples include employee records, "
                           "financial data, and proprietary information.",
            "superTypes": [],
            "attributeDefs": []
        },
        {
            "category": "CLASSIFICATION",
            "name": "Public",
            "description": "Information that is freely available to the public and poses no risk if disclosed. "
                           "Examples include press releases and publicly available reports.",
            "superTypes": [],
            "attributeDefs": []
        },
        {
            "category": "CLASSIFICATION",
            "name": "Internal",
            "description": "Data intended for internal use within the organization. While not highly sensitive, "
                           "it should still be protected to avoid misuse. Examples include internal memos and "
                           "internal project documents.",
            "superTypes": [],
            "attributeDefs": []
        },
        {
            "category": "CLASSIFICATION",
            "name": "Restricted",
            "description": "Highly sensitive data that requires the highest level of protection due to its critical "
                           "nature. Unauthorized access could lead to severe consequences. Examples include trade "
                           "secrets, classified government information, and personal health information (PHI).",
            "superTypes": [],
            "attributeDefs": []
        }
    ],
    "relationshipDefs": [
        {
            "name": "supergraph_object_relationship",
            "description": "Object relationship between 2 supergraph object types",
            "relationshipCategory": "ASSOCIATION",
            "serviceType": "supergraph",
            "attributeDefs": [
                {
                    "name": "field_name",
                    "typeName": "string",
                    "isOptional": False,
                    "isUnique": False,
                    "isIndexable": True
                },
                {
                    "name": "source_keys",
                    "typeName": "array<column>"
                },
                {
                    "name": "target_keys",
                    "typeName": "array<column>"
                }
            ],
            "endDef1": {
                "type": "object_type",
                "name": "relates_to",
                "isContainer": False,
                "cardinality": "SET"
            },
            "endDef2": {
                "type": "object_type",
                "name": "relates_to",
                "isContainer": False,
                "cardinality": "SET"
            },
            "propagateTags": "NONE"
        },
        {
            "name": "supergraph_array_relationship",
            "description": "Array relationship between 2 supergraph object types",
            "relationshipCategory": "ASSOCIATION",
            "serviceType": "supergraph",
            "attributeDefs": [
                {
                    "name": "field_name",
                    "typeName": "string",
                    "isOptional": False,
                    "isUnique": False,
                    "isIndexable": True
                },
                {
                    "name": "source_keys",
                    "typeName": "array<column>"
                },
                {
                    "name": "target_keys",
                    "typeName": "array<column>"
                }
            ],
            "endDef1": {
                "type": "object_type",
                "name": "can be related to many",
                "isContainer": False,
                "cardinality": "SET"
            },
            "endDef2": {
                "type": "object_type",
                "name": "can be traversed into from",
                "isContainer": False,
                "cardinality": "SET"
            },
            "propagateTags": "NONE"
        },
        {
            "name": "data_flow",
            "serviceType": "atlas_core",
            "typeVersion": "1.0",
            "attributeDefs": [
                {
                    "name": "transformation",
                    "typeName": "transform",
                    "isOptional": False
                }
            ],
            "relationshipCategory": "ASSOCIATION",
            "propagateTags": "NONE",
            "endDef1": {
                "type": "supergraph_value",
                "name": "flows to",
                "isContainer": False,
                "cardinality": "SINGLE",
                "isLegacyAttribute": False
            },
            "endDef2": {
                "type": "supergraph_value",
                "name": "sources from",
                "isContainer": False,
                "cardinality": "SINGLE",
                "isLegacyAttribute": False
            }
        },
        {
            "name": "data_source_to_api",
            "serviceType": "atlas_core",
            "typeVersion": "1.0",
            "relationshipCategory": "ASSOCIATION",
            "propagateTags": "NONE",
            "endDef1": {
                "type": "column",
                "name": "flows to",
                "isContainer": False,
                "cardinality": "SINGLE",
                "isLegacyAttribute": False
            },
            "endDef2": {
                "type": "field",
                "name": "comes from",
                "isContainer": False,
                "cardinality": "SINGLE",
                "isLegacyAttribute": False
            }
        },
        {
            "name": "collection_owns_column",
            "serviceType": "atlas_core",
            "typeVersion": "1.0",
            "relationshipCategory": "ASSOCIATION",
            "propagateTags": "NONE",
            "attributeDefs": [
                {
                    "name": "description",
                    "typeName": "string",
                    "isOptional": True,
                    "cardinality": "SINGLE",
                    "valuesMinCount": 0,
                    "valuesMaxCount": 1,
                    "isUnique": False,
                    "isIndexable": True
                }
            ],
            "endDef1": {
                "type": "collection",
                "name": "has these columns",
                "isContainer": False,
                "cardinality": "SET",
                "isLegacyAttribute": False
            },
            "endDef2": {
                "type": "column",
                "name": "from this collection",
                "isContainer": False,
                "cardinality": "SET",
                "isLegacyAttribute": False
            }
        },
        {
            "name": "object_type_owns_field",
            "serviceType": "atlas_core",
            "typeVersion": "1.0",
            "relationshipCategory": "ASSOCIATION",
            "propagateTags": "NONE",
            "attributeDefs": [
                {
                    "name": "description",
                    "typeName": "string",
                    "isOptional": True,
                    "cardinality": "SINGLE",
                    "valuesMinCount": 0,
                    "valuesMaxCount": 1,
                    "isUnique": False,
                    "isIndexable": True
                }
            ],
            "endDef1": {
                "type": "object_type",
                "name": "has available for selection",
                "isContainer": False,
                "cardinality": "SET",
                "isLegacyAttribute": False
            },
            "endDef2": {
                "type": "field",
                "name": "part of",
                "isContainer": False,
                "cardinality": "SET",
                "isLegacyAttribute": False
            }
        },
        {
            "name": "data_connector_has_collection",
            "serviceType": "atlas_core",
            "typeVersion": "1.0",
            "relationshipCategory": "ASSOCIATION",
            "propagateTags": "NONE",
            "attributeDefs": [
                {
                    "name": "description",
                    "typeName": "string",
                    "isOptional": True,
                    "cardinality": "SINGLE",
                    "valuesMinCount": 0,
                    "valuesMaxCount": 1,
                    "isUnique": False,
                    "isIndexable": True
                }
            ],
            "endDef1": {
                "type": "data_connector",
                "name": "sources collection",
                "isContainer": False,
                "cardinality": "SET",
                "isLegacyAttribute": False
            },
            "endDef2": {
                "type": "collection",
                "name": "belongs to data connection",
                "isContainer": False,
                "cardinality": "SET",
                "isLegacyAttribute": False
            }
        },
        {
            "name": "query_has_object_type",
            "serviceType": "atlas_core",
            "typeVersion": "1.0",
            "relationshipCategory": "ASSOCIATION",
            "propagateTags": "NONE",
            "attributeDefs": [
                {
                    "name": "description",
                    "typeName": "string",
                    "isOptional": True,
                    "cardinality": "SINGLE",
                    "valuesMinCount": 0,
                    "valuesMaxCount": 1,
                    "isUnique": False,
                    "isIndexable": True
                }
            ],
            "endDef1": {
                "type": "query",
                "name": "provisions",
                "isContainer": False,
                "cardinality": "SET",
                "isLegacyAttribute": False
            },
            "endDef2": {
                "type": "object_type",
                "name": "is provisioned by",
                "isContainer": False,
                "cardinality": "SET",
                "isLegacyAttribute": False
            }
        },
        {
            "name": "subgraph_has_data_connector",
            "serviceType": "atlas_core",
            "typeVersion": "1.0",
            "relationshipCategory": "ASSOCIATION",
            "propagateTags": "NONE",
            "attributeDefs": [
                {
                    "name": "description",
                    "typeName": "string",
                    "isOptional": True,
                    "cardinality": "SINGLE",
                    "valuesMinCount": 0,
                    "valuesMaxCount": 1,
                    "isUnique": False,
                    "isIndexable": True
                }
            ],
            "endDef1": {
                "type": "subgraph",
                "name": "has data connector",
                "isContainer": False,
                "cardinality": "SET",
                "isLegacyAttribute": False
            },
            "endDef2": {
                "type": "data_connector",
                "name": "belongs to domain",
                "isContainer": False,
                "cardinality": "SET",
                "isLegacyAttribute": False
            }
        },
        {
            "name": "subgraph_has_query",
            "serviceType": "atlas_core",
            "typeVersion": "1.0",
            "relationshipCategory": "ASSOCIATION",
            "propagateTags": "NONE",
            "attributeDefs": [
                {
                    "name": "description",
                    "typeName": "string",
                    "isOptional": True,
                    "cardinality": "SINGLE",
                    "valuesMinCount": 0,
                    "valuesMaxCount": 1,
                    "isUnique": False,
                    "isIndexable": True
                }
            ],
            "endDef1": {
                "type": "subgraph",
                "name": "has query",
                "isContainer": False,
                "cardinality": "SET",
                "isLegacyAttribute": False
            },
            "endDef2": {
                "type": "query",
                "name": "belongs to domain",
                "isContainer": False,
                "cardinality": "SET",
                "isLegacyAttribute": False
            }
        },
        {
            "name": "subgraph_has_mutation",
            "serviceType": "atlas_core",
            "typeVersion": "1.0",
            "relationshipCategory": "ASSOCIATION",
            "propagateTags": "NONE",
            "attributeDefs": [
                {
                    "name": "description",
                    "typeName": "string",
                    "isOptional": True,
                    "cardinality": "SINGLE",
                    "valuesMinCount": 0,
                    "valuesMaxCount": 1,
                    "isUnique": False,
                    "isIndexable": True
                }
            ],
            "endDef1": {
                "type": "subgraph",
                "name": "has mutation",
                "isContainer": False,
                "cardinality": "SET",
                "isLegacyAttribute": False
            },
            "endDef2": {
                "type": "mutation",
                "name": "belongs to domain",
                "isContainer": False,
                "cardinality": "SET",
                "isLegacyAttribute": False
            }
        },
        {
            "name": "subgraph_has_object_type",
            "serviceType": "atlas_core",
            "typeVersion": "1.0",
            "relationshipCategory": "ASSOCIATION",
            "propagateTags": "NONE",
            "attributeDefs": [
                {
                    "name": "description",
                    "typeName": "string",
                    "isOptional": True,
                    "cardinality": "SINGLE",
                    "valuesMinCount": 0,
                    "valuesMaxCount": 1,
                    "isUnique": False,
                    "isIndexable": True
                }
            ],
            "endDef1": {
                "type": "subgraph",
                "name": "owns object",
                "isContainer": False,
                "cardinality": "SET",
                "isLegacyAttribute": False
            },
            "endDef2": {
                "type": "object_type",
                "name": "belongs to domain",
                "isContainer": False,
                "cardinality": "SET",
                "isLegacyAttribute": False
            }
        },
        {
            "name": "supergraph_has_subgraph",
            "serviceType": "atlas_core",
            "typeVersion": "1.0",
            "relationshipCategory": "ASSOCIATION",
            "propagateTags": "NONE",
            "attributeDefs": [
                {
                    "name": "description",
                    "typeName": "string",
                    "isOptional": True,
                    "cardinality": "SINGLE",
                    "valuesMinCount": 0,
                    "valuesMaxCount": 1,
                    "isUnique": False,
                    "isIndexable": True
                }
            ],
            "endDef1": {
                "type": "supergraph",
                "name": "owns",
                "isContainer": False,
                "cardinality": "SINGLE",
                "isLegacyAttribute": False
            },
            "endDef2": {
                "type": "subgraph",
                "name": "belongs_to",
                "isContainer": False,
                "cardinality": "SET",
                "isLegacyAttribute": False
            }
        }
    ],
    "businessMetadataDefs": [
        {
            "name": "data_analysis",
            "displayName": "Data Analysis",
            "description": "Business metadata for Fields and Columns",
            "typeVersion": "1.0",
            "attributeDefs": [
                {
                    "name": "businessNames",
                    "displayName": "Business Names",
                    "typeName": "array<string>",
                    "isOptional": True,
                    "isUnique": True,
                    "isIndexable": True,
                    "options": {
                        "maxStrLength": 250,
                        "applicableEntityTypes": "[\"field\",\"object_type\",\"column\",\"query\",\"collection\",\"subgraph\",\"supergraph\"]"
                    }
                },
                {
                    "name": "testingType",
                    "displayName": "Testing Type",
                    "typeName": "testingTypeEnumName",
                    "isOptional": True,
                    "isUnique": True,
                    "isIndexable": True,
                    "options": {
                        "maxStrLength": 250,
                        "applicableEntityTypes": "[\"field\",\"column\"]"
                    }
                },
                {
                    "name": "validValues",
                    "displayName": "Valid Values",
                    "typeName": "array<string>",
                    "isOptional": True,
                    "isUnique": True,
                    "isIndexable": True,
                    "options": {
                        "maxStrLength": 250,
                        "applicableEntityTypes": "[\"field\",\"column\"]"
                    }
                }
            ]
        },
        {
            "name": "data_governance",
            "displayName": "Data Governance",
            "description": "Business metadata for Fields and Columns",
            "typeVersion": "1.0",
            "attributeDefs": [
                {
                    "name": "dataSteward",
                    "displayName": "Data Steward",
                    "typeName": "string",
                    "isOptional": True,
                    "isUnique": True,
                    "isIndexable": True,
                    "options": {
                        "maxStrLength": 250,
                        "applicableEntityTypes": "[\"subgraph\",\"supergraph\"]"
                    }
                }
            ]
        }
    ],
    "entityDefs": [
        {
            "name": "transform",
            "category": "ENTITY",
            "typeVersion": "1.0",
            "serviceType": "supergraph",
            "superTypes": [
                "Asset"
            ],
            "description": "A definition of a transformation",
            "attributeDefs": [
                {
                    "name": "transformation",
                    "typeName": "transform_enum",
                    "isOptional": False
                },
                {
                    "name": "explanation",
                    "typeName": "string",
                    "isOptional": True
                }
            ]
        },
        {
            "name": "column",
            "category": "ENTITY",
            "typeVersion": "1.0",
            "superTypes": [
                "Asset"
            ],
            "serviceType": "supergraph",
            "description": "A column definition within a table or view",
            "attributeDefs": [
                {
                    "name": "dataType",
                    "typeName": "scalar"
                },
                {
                    "name": "nullable",
                    "typeName": "boolean"
                }
            ]
        },
        {
            "name": "foreign_key",
            "category": "ENTITY",
            "typeVersion": "1.0",
            "superTypes": [
                "Asset"
            ],
            "serviceType": "supergraph",
            "description": "A foreign key definition within a collection",
            "attributeDefs": [
                {
                    "name": "column_mapping",
                    "typeName": "map<column,column>",
                    "isOptional": True
                },
                {
                    "name": "foreign_collection",
                    "typeName": "collection",
                    "isOptional": True
                }
            ]
        },
        {
            "name": "collection",
            "category": "ENTITY",
            "typeVersion": "1.0",
            "superTypes": [
                "Asset"
            ],
            "serviceType": "supergraph",
            "description": "A collection definition",
            "attributeDefs": [
                {
                    "name": "primary_keys",
                    "typeName": "array<string>"
                },
                {
                    "name": "foreign_keys",
                    "typeName": "array<foreign_key>",
                    "isOptional": True
                },
                {
                    "name": "columns",
                    "typeName": "array<column>"
                }
            ]
        },
        {
            "name": "supergraph_value",
            "category": "ENTITY",
            "description": "A data connector scalar definition",
            "superTypes": [
                "Asset"
            ],
            "serviceType": "supergraph"
        },
        {
            "name": "scalar",
            "category": "ENTITY",
            "description": "A data connector scalar definition",
            "superTypes": [
                "supergraph_value"
            ],
            "serviceType": "supergraph",
            "attributeDefs": [
                {
                    "name": "representation",
                    "typeName": "string"
                },
                {
                    "name": "aggregate_functions",
                    "typeName": "array<string>"
                },
                {
                    "name": "comparison_operators",
                    "typeName": "array<string>"
                }
            ]
        },
        {
            "name": "data_connector",
            "category": "ENTITY",
            "description": "Represents a single data connector",
            "superTypes": [
                "Asset"
            ],
            "serviceType": "supergraph",
            "attributeDefs": [
                {
                    "name": "subgraph",
                    "typeName": "string",
                    "isIndexable": True
                },
                {
                    "name": "source",
                    "typeName": "string",
                    "isIndexable": True
                },
                {
                    "name": "env_mapping",
                    "typeName": "map<string,map<string,string>>",
                    "isIndexable": True
                },
                {
                    "name": "scalar_types",
                    "typeName": "array<scalar>"
                },
                {
                    "name": "functions",
                    "typeName": "array<string>"
                },
                {
                    "name": "procedures",
                    "typeName": "array<string>"
                },
                {
                    "name": "collections",
                    "typeName": "array<collection>"
                }
            ]
        },
        {
            "name": "field",
            "category": "ENTITY",
            "typeVersion": "1.0",
            "superTypes": [
                "Asset"
            ],
            "serviceType": "supergraph",
            "description": "A field definition within an object type",
            "attributeDefs": [
                {
                    "name": "primitive",
                    "typeName": "string",
                    "isOptional": False,
                    "isUnique": False,
                    "isIndexable": True
                },
                {
                    "name": "column",
                    "typeName": "column"
                }
            ]
        },
        {
            "name": "object_type",
            "category": "ENTITY",
            "serviceType": "supergraph",
            "description": "Represents a subgraph object type",
            "superTypes": [
                "supergraph_value"
            ],
            "attributeDefs": [
                {
                    "name": "fields",
                    "typeName": "array<field>"
                }
            ]
        },
        {
            "name": "mutation_result",
            "category": "ENTITY",
            "serviceType": "supergraph",
            "description": "Represents a mutation result",
            "superTypes": [
                "Asset"
            ]
        },
        {
            "name": "query",
            "category": "ENTITY",
            "description": "Represents a query",
            "superTypes": [
                "Asset"
            ],
            "serviceType": "supergraph",
            "typeVersion": "1.0",
            "attributeDefs": [
                {
                    "name": "record",
                    "typeName": "object_type"
                },
                {
                    "name": "is_array",
                    "typeName": "boolean"
                },
                {
                    "name": "unique_identifiers",
                    "typeName": "array<string>"
                }
            ]
        },
        {
            "name": "mutation",
            "category": "ENTITY",
            "description": "Represents a single mutation",
            "superTypes": [
                "Asset"
            ],
            "serviceType": "supergraph",
            "typeVersion": "1.0",
            "attributeDefs": [
                {
                    "name": "data",
                    "typeName": "mutation_result"
                }
            ]
        },
        {
            "name": "subgraph",
            "category": "ENTITY",
            "description": "Represents a Subgraph",
            "superTypes": [
                "Asset"
            ],
            "serviceType": "supergraph",
            "typeVersion": "1.0",
            "attributeDefs": [
                {
                    "name": "queries",
                    "typeName": "array<query>"
                },
                {
                    "name": "mutations",
                    "typeName": "array<mutation>"
                },
                {
                    "name": "data_connectors",
                    "typeName": "array<data_connector>"
                },
                {
                    "name": "subgraph_schema",
                    "typeName": "array<object_type>"
                }
            ]
        },
        {
            "name": "supergraph",
            "category": "ENTITY",
            "description": "Represents a Supergraph",
            "superTypes": [
                "Asset"
            ],
            "serviceType": "supergraph",
            "typeVersion": "1.0",
            "attributeDefs": [
                {
                    "name": "domains",
                    "typeName": "array<subgraph>",
                    "isIndexable": True
                },
                {
                    "name": "console_url",
                    "typeName": "string",
                    "isIndexable": True
                },
                {
                    "name": "graphql_service_url",
                    "typeName": "string",
                    "isIndexable": True
                },
                {
                    "name": "grpc_service_url",
                    "typeName": "string",
                    "isIndexable": True
                },
                {
                    "name": "openapi_ui_url",
                    "typeName": "string",
                    "isIndexable": True
                },
                {
                    "name": "openapi_service_url",
                    "typeName": "string",
                    "isIndexable": True
                }
            ]
        }
    ]
}
